@@ -6,16 +6,78 @@
 Summary
 -------
 Exports and converts *waf* project data, for C/C++ programs, static- and shared
-libraries, into **Eclipse** *CDT* project files (.cbp) and workspaces 
-(codeblock.workspace).
-**Eclipse** is an open source integrated development environment, which can be, 
-amongst others, used for development of C/C++ programs. 
+libraries, into **Eclipse** *CDT* project files (.projects, .cproject).
+**Eclipse** is an open source integrated development environment, which can be 
+used, amongst others, for development of C/C++ programs. 
 
 See https://www.eclipse.org and https://www.eclipse.org/cdt for a more detailed 
 description on how to install and use it for your particular Desktop environment.
 
-REMARKS:
-Supports export of C/C++ projects using GCC / MinGW only. CygWin is NOT supported!
+.. note::
+	Export of C/C++ projects are supported for GCC/MinGW compilers only, 
+	other compilers and/or environments (e.g. MSVC, CygWin) are NOT supported!
+
+Description
+-----------
+For each single C/C++ task generator (*waflib.TaskGenerator*), for instance a 
+*bld.program(...)* which has been defined within a *wscript* file somewhere in
+the build environment, a single **Eclipse** *CDT* project file will be generated
+in the same directory as where the task generator has been defined. Once generated
+these projects can be imported into **Eclipse** using 'Import -> General ->
+Existing Projects Into Workspace'.
+
+Example below presents an overview of an environment in which **Eclipse** 
+projects already have been exported::
+
+        .
+        ├── components
+        │   └── clib
+        │       ├── program
+        │       │   ├── .project
+        │       │   ├── .cproject
+        │       │   └── wscript
+        │       ├── shared
+        │       │   ├── .project
+        │       │   ├── .cproject
+        │       │   └── wscript
+        │       └── static
+        │           ├── .project
+        │           ├── .cproject
+        │           └── wscript
+        │
+        └── wscript
+
+
+.. warning:: 
+	Export of multiple C/C++ build tasks (program, stlib and/or shlib) 
+	which have been defined within the same directory is not supported.
+
+Projects will be exported such that they will use the same settings and 
+structure as has been defined for that build task within the *waf* build 
+environment. Projects will contain exactly **one** build target per build 
+variant that has been defined in the *waf* build environment, as explained in
+the example below;
+
+    **input**:
+    In a *waf* build environment three variants have been defined, one 
+    default (without name) build used for normal compiling and linking for the 
+    current host, and two variants used for cross compiling and linking for
+    embedded systems; one is named *arm5* the other *arm7*.
+    Also the *complete* environment has been configured to be build with
+    debugging information (i.e. the CFLAGS and CXXFLAGS both contain the 
+    compiler option`-g`).
+
+    **output**:
+    Each exported project will contain the following build targets:
+    - The first named **debug**, for the current host platform,
+    - The second named **arm5-debug**, for the ARM5 target, and
+    - The third named **arm7-debug**, for the ARM7 target. 
+
+Please note that in contrast to a *normal* IDE setup the exported projects 
+will contain either a *debug* **or** a *release* build target but not both at
+the same time. By doing so exported projects will always use the same settings
+(e.g. compiler options, installation paths) as when building the same task in
+the *waf* build environment from command line.
 
 Usage
 -----
@@ -30,16 +92,12 @@ removed using the *clean* command, as shown in the example below::
         $ waf eclipse --clean
 '''
 
-# TODO: add detailed description for 'eclipse' module
-# TODO: add support for multiple variant (e.g. cross-compile)
-
 
 import sys
 import os
 import codecs
 import xml.etree.ElementTree as ElementTree
 from xml.dom import minidom
-import waflib
 from waflib import Utils, Logs, Errors, Context
 from waflib.Build import BuildContext
 
@@ -158,14 +216,14 @@ def cleanup(bld):
 		if targets and tgen.get_name() not in targets:
 			continue
 		if set(('c', 'cxx')) & set(getattr(tgen, 'features', [])):
-			Project(bld, tgen).clean()
-			CDTProject(bld, tgen).clean()
+			Project(bld, tgen).cleanup()
+			CDTProject(bld, tgen).cleanup()
 
 
 def detect_project_duplicates(bld):
 	'''Warns when multiple TaskGen's have been defined in the same directory.
 
-	Since Eclipse works with static project filenames, only one project	per
+	Since Eclipse works with static project filenames, only one project per
 	directory can be created. If multiple task generators have been defined
 	in the same directory (i.e. same wscript) one will overwrite the other(s).
 	This problem can only e circumvented by changing the structure of the 
@@ -207,8 +265,8 @@ class EclipseProject(object):
 	:type bld: waflib.Build.BuildContext
 
 	:param tgen: Task generator that contains all information of the task to be
-				converted and exported to the *Eclipse* project.
-	:type tgen:	waflib.Task.TaskGen
+	             converted and exported to the *Eclipse* project.
+	:type tgen: waflib.Task.TaskGen
 	'''
 	def __init__(self, bld, tgen, fname, template):
 		self.bld = bld
@@ -275,8 +333,8 @@ class Project(EclipseProject):
 	:type bld: waflib.Build.BuildContext
 
 	:param tgen: Task generator that contains all information of the task to be
-				converted and exported to the *Eclipse* project.
-	:type tgen:	waflib.Task.TaskGen
+	             converted and exported to the *Eclipse* project.
+	:type tgen: waflib.Task.TaskGen
 	'''
 	def __init__(self, bld, tgen):
 		super(Project, self).__init__(bld, tgen, '.project', ECLIPSE_PROJECT)
@@ -291,7 +349,7 @@ class Project(EclipseProject):
 
 	def add_projects(self, root):
 		projects = root.find('projects')
-		uses = list(Utils.to_list(getattr(self.tgen, 'use', [])))		
+		uses = list(Utils.to_list(getattr(self.tgen, 'use', [])))
 		for project in projects.findall('project'):
 			if project.text in uses: uses.remove(project.text)
 		for use in uses:
@@ -327,85 +385,91 @@ class CDTProject(EclipseProject):
 	:type bld: waflib.Build.BuildContext
 
 	:param tgen: Task generator that contains all information of the task to be
-				converted and exported to the *Eclipse* project.
-	:type tgen:	waflib.Task.TaskGen
-		
+	             converted and exported to the *Eclipse* project.
+	:type tgen: waflib.Task.TaskGen
+	
 	:param project: Reference to *Eclipse* project (which will export the 
-					*.project* file.
+	                *.project* file.
 	:param project: Project
 	'''
-	def __init__(self, bld, tgen):		
+	def __init__(self, bld, tgen):
 		super(CDTProject, self).__init__(bld, tgen, '.cproject', ECLIPSE_CDT_PROJECT)
-		
 		d = tgen.env.DEST_OS
 		c = tgen.env.DEST_CPU
-		
+		l = 'cpp' if 'cxx' in tgen.features else 'c'
+		b = 'debug' if '-g' in tgen.env.CFLAGS else 'release'
+
+		self.cross = not sys.platform.startswith(d) or c not in ('x86_64', 'x86', 'ia') # TODO: assuming host is intel based only!!!
 		self.comments = ['<?xml version="1.0" encoding="UTF-8" standalone="no"?>','<?fileVersion 4.0.0?>']
-		self.language = 'cpp' if 'cxx' in tgen.features else 'c'
+		self.language = l
 		self.cdt = {}
 		if set(('cprogram', 'cxxprogram')) & set(tgen.features):
 			self.cdt['ext'] = 'exe'
 			self.cdt['kind'] = 'Executable'
 			self.cdt['buildArtefactType'] = 'org.eclipse.cdt.build.core.buildArtefactType.exe'
-			if d!='win32':
-				self.cdt['artifactExtension'] = ''
+			self.cdt['artifactExtension'] = 'exe' if d=='win32' else ''
 
 		if set(('cshlib', 'cxxshlib')) & set(tgen.features):
 			self.cdt['ext'] = 'so'
 			self.cdt['kind'] = 'Shared Library'
 			self.cdt['buildArtefactType'] = 'org.eclipse.cdt.build.core.buildArtefactType.sharedLib'
 			self.cdt['artifactExtension'] = 'dll' if d=='win32' else 'so'
-			
+
 		if set(('cstlib', 'cxxstlib')) & set(tgen.features):
 			self.cdt['ext'] = 'lib'
 			self.cdt['kind'] = 'Static Library'
 			self.cdt['buildArtefactType'] = 'org.eclipse.cdt.build.core.buildArtefactType.staticLib'
 			self.cdt['artifactExtension'] = 'a'
 
-		ar = os.path.splitext(os.path.basename(tgen.env.AR))[0]	
-		self.cdt['c'] = os.path.splitext(os.path.basename(tgen.env.CC[0]))[0]
+		e = self.cdt['ext']
+		ar = os.path.splitext(os.path.basename(tgen.env.AR))[0]
+		cc = tgen.env.CC[0]
+		self.cdt['c'] = os.path.splitext(os.path.basename(cc))[0]
 		self.cdt['cpp'] = os.path.splitext(os.path.basename(tgen.env.CXX[0]))[0]
 		self.cdt['ar'] = ar
 		self.cdt['as'] = ar.replace('ar', 'as')
+		self.cdt['cc_path'] = os.path.dirname(cc).replace('\\', '/')
 
-		self.cdt['gnu'] = 'gnu%s' % ('.mingw' if sys.platform == 'win32' else '')
-		self.cdt['build'] = 'debug' if '-g' in tgen.env.CFLAGS else 'release'
-		self.cdt['parent'] = 'cdt.managedbuild.config.%s.%s.%s' % (self.cdt['gnu'], self.cdt['ext'], self.cdt['build'])
+		if sys.platform=='win32':
+			gnu = 'gnu.mingw'
+			gnut = 'mingw'
+		elif self.cross:
+			gnu = 'gnu.cross'
+			gnut = 'cross'
+		else:
+			gnu = 'gnu'
+			gnut = None
+
+		self.cdt['gnut'] = gnut
+		self.cdt['gnu'] = gnu
+		self.cdt['build'] = b
+		self.cdt['parent'] = 'cdt.managedbuild.config.%s.%s.%s' % (gnu, self.cdt['ext'], b)
 		self.cdt['instance'] = '%s.%s' % (self.cdt['parent'], self.get_uuid())
-		self.cdt['name'] = '%s_%s' % (tgen.env.DEST_OS, tgen.env.DEST_CPU)
-		self.cdt['parser'] = 'org.eclipse.cdt.core.PE' if tgen.env.DEST_OS=='win32' else 'org.eclipse.cdt.core.ELF' 		
+		self.cdt['name'] = '%s%s' % (b, '-%s' % bld.variant if bld.variant else '')
 
-		s = 'cdt.managedbuild.tool.gnu.%s.compiler.%s' % (self.language, 'mingw.' if sys.platform=='win32' else '')
-		s += '%s.%s' % (self.cdt['ext'], 'debug' if '-g' in tgen.env.CFLAGS else 'release')
-		self.cdt['compiler'] = '%s.%s' % (s, self.get_uuid())	
-		self.cdt['input'] = 'cdt.managedbuild.tool.gnu.%s.compiler.input.%s' % (self.language, self.get_uuid())
-		self.cdt['archiver'] = 'cdt.managedbuild.tool.gnu.archiver%s.base' % ('.mingw' if d=='win32' else '')
+		s = 'cdt.managedbuild.tool.gnu.%s.compiler.%s%s.%s' % (l, '%s.' % gnut if gnut else '', e, b)
+		self.cdt['compiler'] = '%s.%s' % (s, self.get_uuid())
+		self.cdt['input'] = 'cdt.managedbuild.tool.gnu.%s.compiler.input.%s' % (l, self.get_uuid())
+
+		t = '%s.%s' % (e, b)
+		tt = '%s.%s' % ('.%s' % gnut if gnut else '', t)
+		self.cdt['toolchain'] = 'cdt.managedbuild.toolchain.%s.%s' % (gnu, t)
+		self.cdt['platform'] = 'cdt.managedbuild.target.gnu.platform%s' % (tt)
+		self.cdt['compiler_option'] = 'gnu.{0}.compiler.option.{1}.level'
+		self.cdt['archiver'] = 'cdt.managedbuild.tool.gnu.archiver%s.base' % ('.mingw' if sys.platform=='win32' else '')
 		
-		# TODO: assuming host is i386 only!!!		
-		if sys.platform.startswith(d) and c in ('x86_64', 'x86', 'ia'):
-			self.cross = False
-			t = '%s.%s' % (self.cdt['ext'], self.cdt['build'])
-			tt = '%s.%s' % ('.mingw' if d=='win32' else '', t)
-			self.cdt['toolchain'] = 'cdt.managedbuild.toolchain.%s.%s' % (self.cdt['gnu'], t)
-			self.cdt['platform'] = 'cdt.managedbuild.target.gnu.platform%s' % (tt)
-			self.cdt['assembler'] = 'cdt.managedbuild.tool.gnu.assembler%s' % (tt)
+		if self.cross and sys.platform!='win32':
+			self.cdt['c_compiler'] = 'cdt.managedbuild.tool.gnu.cross.c.compiler'
+			self.cdt['cpp_compiler'] = 'cdt.managedbuild.tool.gnu.cross.cpp.compiler'
+			self.cdt['c_linker'] = 'cdt.managedbuild.tool.gnu.cross.c.linker'
+			self.cdt['cpp_linker'] = 'cdt.managedbuild.tool.gnu.cross.cpp.linker'
+			self.cdt['assembler'] = 'cdt.managedbuild.tool.gnu.cross.assembler'
+		else:
 			self.cdt['c_compiler'] = 'cdt.managedbuild.tool.gnu.c.compiler%s' % (tt)
 			self.cdt['cpp_compiler'] = 'cdt.managedbuild.tool.gnu.cpp.compiler%s' % (tt)
 			self.cdt['c_linker'] = 'cdt.managedbuild.tool.gnu.c.linker%s' % (tt)
-			self.cdt['cpp_linker'] = 'cdt.managedbuild.tool.gnu.cpp.linker%s' % (tt)			
-			self.cdt['compiler_option'] = 'gnu.{0}.compiler%s.%s.%s.option.{1}.level' % ('.mingw' if d=='win32' else '', self.cdt['ext'], self.cdt['build'])
-		else:
-			self.cross = True
-			self.cdt['toolchain'] = 'cdt.managedbuild.toolchain.base'
-			self.cdt['platform'] = 'cdt.managedbuild.target.gnu.platforms.base'
-			self.cdt['archList'] = 'all'
-			self.cdt['osList'] = 'linux,hpux,aix,qnx'
-			self.cdt['assembler'] = 'cdt.managedbuild.tool.gnu.assembler.base'
-			self.cdt['c_compiler'] = 'cdt.managedbuild.tool.gnu.c.compiler.base'
-			self.cdt['cpp_compiler'] = 'cdt.managedbuild.tool.gnu.cpp.compiler.base'
-			self.cdt['c_linker'] = 'cdt.managedbuild.tool.gnu.c.linker.base'
-			self.cdt['cpp_linker'] = 'cdt.managedbuild.tool.gnu.cpp.linker.base'
-
+			self.cdt['cpp_linker'] = 'cdt.managedbuild.tool.gnu.cpp.linker%s' % (tt)
+			self.cdt['assembler'] = 'cdt.managedbuild.tool.gnu.assembler%s' % (tt)
 
 	def get_uuid(self):
 		uuid = codecs.encode(os.urandom(4), 'hex_codec')
@@ -429,7 +493,7 @@ class CDTProject(EclipseProject):
 
 	def update_cdt_core_settings(self, module):
 		cconfig = self.cconfig_get(module)
-		if not cconfig:
+		if cconfig is None:
 			cconfig = ElementTree.fromstring(ECLIPSE_CDT_CCONFIGURATION)
 			module.append(cconfig)
 		self.cconfig_update(cconfig)
@@ -516,30 +580,52 @@ class CDTProject(EclipseProject):
 		self.cconfig_toolchain_update(folder)
 
 	def	cconfig_toolchain_update(self, folder):
-		d = self.tgen.env.DEST_OS
 		toolchain = folder.find('toolChain')
 		toolchain.set('superClass', self.cdt['toolchain'])
 		toolchain.set('id', '%s.%s' % (self.cdt['toolchain'], self.get_uuid()))
-		toolchain.set('name', '%s' % ('MinGW GCC' if d=='win32' else 'Linux GCC'))
+		name = 'MinGW GCC' if sys.platform=='win32' else 'Linux GCC'
+		if self.cross and sys.platform!='win32':		
+			name = 'Cross GCC'
+		toolchain.set('name', name)
 
+		self.toolchain_cross_update(toolchain)
 		self.toolchain_target_update(toolchain)
 		self.toolchain_builder_update(toolchain)
-		self.toolchain_assembler_update(toolchain)
-		self.toolchain_archiver_update(toolchain)
-		self.toolchain_compiler_update(toolchain, 'cpp')
 		self.toolchain_compiler_update(toolchain, 'c')
-		self.toolchain_linker_update(toolchain, 'cpp')
+		self.toolchain_compiler_update(toolchain, 'cpp')
 		self.toolchain_linker_update(toolchain, 'c')
+		self.toolchain_linker_update(toolchain, 'cpp')
+		self.toolchain_archiver_update(toolchain)
+		self.toolchain_assembler_update(toolchain)
 
-	def toolchain_target_update(self, toolchain):		
+	def toolchain_cross_update(self, toolchain):
+		options = toolchain.findall('option')
+		for option in options:
+			toolchain.remove(option)
+		if not self.cross or sys.platform=='win32':
+			return
+
+		option = ElementTree.Element('option', {'valueType':'string', 'name':'Path'})
+		option.set('superClass', 'cdt.managedbuild.option.gnu.cross.path')
+		option.set('id', 'cdt.managedbuild.option.gnu.cross.path.%s' % self.get_uuid())
+		option.set('value', self.cdt['cc_path'])
+		toolchain.insert(0, option)
+
+		option = ElementTree.Element('option', {'valueType':'string', 'name':'Prefix'})
+		option.set('superClass', 'cdt.managedbuild.option.gnu.cross.prefix')
+		option.set('id', 'cdt.managedbuild.option.gnu.cross.prefix.%s' % self.get_uuid())
+		option.set('value', str(self.cdt['c']).rstrip('gcc'))
+		toolchain.insert(0, option)
+
+	def toolchain_target_update(self, toolchain):
 		target = toolchain.find('targetPlatform')
 		target.set('name', self.cdt['name'])
 		target.set('superClass', self.cdt['platform'])
 		target.set('id', '%s.%s' % (self.cdt['platform'], self.get_uuid()))
 		if self.cross:
-			target.set('archList', self.cdt['archList'])
-			target.set('osList', self.cdt['osList'])
-		target.set('binaryParser', 'org.eclipse.cdt.core.PE;org.eclipse.cdt.core.ELF;org.eclipse.cdt.core.GNU_ELF')
+			target.set('archList', 'all')
+			target.set('osList', 'all')
+		target.set('binaryParser', 'org.eclipse.cdt.core.PE;org.eclipse.cdt.core.ELF')
 
 	def toolchain_builder_update(self, toolchain):
 		builder = toolchain.find('builder')
@@ -547,7 +633,7 @@ class CDTProject(EclipseProject):
 		builder.set('superClass', 'cdt.managedbuild.target.gnu.builder.base')
 		builder.set('id', '%s.%s' % (builder.get('superClass'), self.get_uuid()))
 		builder.set('keepEnvironmentInBuildfile', 'false')
-		builder.set('name', 'Gnu Make Builder%s' % ('.%s' % self.cdt['name'] if self.cross else ''))
+		builder.set('name', 'Gnu Make Builder')
 
 	def toolchain_archiver_get(self, toolchain):
 		for tool in toolchain.findall('tool'):
@@ -560,9 +646,9 @@ class CDTProject(EclipseProject):
 		archiver = self.toolchain_archiver_get(toolchain)
 		archiver.set('superClass', self.cdt['archiver'])
 		archiver.set('id', '%s.%s' % (self.cdt['archiver'], self.get_uuid()))
-		archiver.set('name', 'GCC Archiver')
-		if self.cross:
+		if self.cross and sys.platform=='win32':
 			archiver.set('command', self.cdt['ar'])
+		archiver.set('name', 'GCC Archiver')
 
 	def toolchain_assembler_get(self, toolchain):
 		for tool in toolchain.findall('tool'):
@@ -574,9 +660,12 @@ class CDTProject(EclipseProject):
 	def toolchain_assembler_update(self, toolchain):
 		assembler = self.toolchain_assembler_get(toolchain)
 		assembler.set('superClass', self.cdt['assembler'])
-		assembler.set('id', '%s.%s' % (self.cdt['assembler'], self.get_uuid()))		
-		assembler.set('name', 'GCC Assembler')
+		assembler.set('id', '%s.%s' % (self.cdt['assembler'], self.get_uuid()))
+		name = 'MinGW Assembler' if sys.platform=='win32' else 'GCC Assembler'
 		if self.cross:
+			name = 'Cross GCC Assembler'
+		assembler.set('name', name)
+		if self.cross and sys.platform=='win32':
 			assembler.set('command', self.cdt['as'])
 
 		inputtype = assembler.find('inputType')
@@ -596,15 +685,18 @@ class CDTProject(EclipseProject):
 		compiler = self.toolchain_compiler_get(toolchain, language)
 		compiler.set('superClass', self.cdt['%s_compiler' % language])
 		compiler.set('id', '%s.%s' % (compiler.get('superClass'), self.get_uuid()))
-		compiler.set('name', 'GCC %s Compiler' % language.upper().replace('P', '+'))
+		name = 'G++ Compiler' if language=='cpp' else 'GCC Compiler'
 		if self.cross:
+			name = 'Cross ' + name
+		compiler.set('name', name)
+		if self.cross and sys.platform=='win32':
 			compiler.set('command', self.cdt[language])
-		self.compiler_add_options(compiler, language)
 		self.compiler_add_includes(compiler, language)
 		self.compiler_add_defines(compiler, language)
+		self.compiler_add_options(compiler, language)
 		self.compiler_add_input(compiler, language)
 
-	def compiler_add_options(self, compiler, language):		
+	def compiler_add_options(self, compiler, language):
 		if self.cdt['build'] == 'debug':
 			optimization_level = 'none'
 			debug_level = 'max'
@@ -612,19 +704,20 @@ class CDTProject(EclipseProject):
 			optimization_level = 'most'
 			debug_level = 'none'
 
-		d = self.tgen.env.DEST_OS
-		t = 'gnu.%s.compiler%s' % (language, '.mingw' if d=='win32' else '')
+		gnut = self.cdt['gnut']
+		t = 'gnu.%s.compiler%s' % (language, '.%s' % gnut if gnut else '')
+
 		option = ElementTree.SubElement(compiler, 'option', {'name':'Optimization Level', 'valueType':'enumerated'})
 		option.set('superClass', '%s.%s.%s.option.optimization.level' % (t, self.cdt['ext'], self.cdt['build']))
 		option.set('id', '%s.%s' % (option.get('superClass'), self.get_uuid()))
 		option.set('defaultValue', 'gnu.%s.optimization.level.%s' % (language, optimization_level))
 
 		option = ElementTree.SubElement(compiler, 'option', {'name':'Debug Level', 'valueType':'enumerated'})
-		option.set('superClass', '%s.%s.%s.option.debugging.level' % (t, self.cdt['ext'], self.cdt['build']))		
+		option.set('superClass', '%s.%s.%s.option.debugging.level' % (t, self.cdt['ext'], self.cdt['build']))
 		option.set('id', '%s.%s' % (option.get('superClass'), self.get_uuid()))
 		option.set('value', 'gnu.%s.debugging.level.%s' % (language, debug_level))
 
-		if self.tgen.env.DEST_CPU == 'powerpc':
+		if self.tgen.env.DEST_CPU == 'powerpc': # TODO: only when using ancient 2.95 compiler!
 			option = ElementTree.SubElement(compiler, 'option', {'value':'-c','valueType':'string'})
 			option.set('superClass', 'gnu.c.compiler.option.misc.other')
 			option.set('id', '%s.%s' % (option.get('superClass'), self.get_uuid()))
@@ -649,7 +742,7 @@ class CDTProject(EclipseProject):
 
 		includes = list(set([str(i).lstrip('./') for i in includes]))
 		for include in includes:
-			pth = os.path.join(self.tgen.path.abspath(), include)			
+			pth = os.path.join(self.tgen.path.abspath(), include)
 			if os.path.exists(pth):
 				listoption = ElementTree.SubElement(option, 'listOptionValue', {'builtIn':'false'})
 				listoption.set('value', '"${workspace_loc:/${ProjName}/%s}"' % (include))
@@ -711,12 +804,15 @@ class CDTProject(EclipseProject):
 		linker = self.toolchain_linker_get(toolchain, language)
 		linker.set('superClass', self.cdt['%s_linker' % language])
 		linker.set('id', '%s.%s' % (linker.get('superClass'), self.get_uuid()))
-		linker.set('name', 'GCC %s Linker' % language.upper().replace('P', '+'))
+		name = 'G++ Linker' if language=='cpp' else 'GCC Linker'
 		if self.cross:
-			linker.set('command', self.cdt[language])
-			
+			name = 'Cross ' + name
+		linker.set('name', name)
+		if self.cross and sys.platform=='win32':
+			linker.set('command', self.cdt[language])		
 		if self.cdt['ext'] == 'so':
-			option = ElementTree.SubElement(linker, 'option', {'name':'Shared (-shared)', 'defaultValue':'true', 'valueType':'boolean'})
+			option = ElementTree.SubElement(linker, 'option', {'defaultValue':'true', 'valueType':'boolean'})
+			option.set('name', 'Shared (-shared)')
 			option.set('superClass', 'gnu.%s.link.so.%s.option.shared' % (language, self.cdt['build']))
 			option.set('id', '%s.%s' % (option.get('superClass'), self.get_uuid()))
 
@@ -806,7 +902,7 @@ ECLIPSE_CDT_PROJECT = \
 	<storageModule moduleId="org.eclipse.cdt.core.settings">
 	</storageModule>
 	<storageModule moduleId="cdtBuildSystem" version="4.0.0">
-		<project id="" name="" projectType=""/>	
+		<project id="" name="" projectType=""/>
 	</storageModule>
 	<storageModule moduleId="scannerConfiguration">
 		<autodiscovery enabled="true" problemReportingEnabled="true" selectedProfileId=""/>
@@ -850,7 +946,7 @@ ECLIPSE_CDT_CCONFIGURATION = '''
 			</folderInfo>
 		</configuration>
 	</storageModule>
-	<storageModule moduleId="org.eclipse.cdt.core.externalSettings"/>	
+	<storageModule moduleId="org.eclipse.cdt.core.externalSettings"/>
 </cconfiguration>
 '''
 
